@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2018 DreamWorks Animation LLC
 //
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
@@ -34,11 +34,12 @@
 #ifndef OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 #define OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 
-#include <math.h>
-#include <iostream>
-#include <boost/format.hpp>
-#include <openvdb/Exceptions.h>
 #include "Math.h"
+#include <openvdb/Exceptions.h>
+#include <algorithm> // for std::max()
+#include <cmath>
+#include <iostream>
+#include <string>
 
 
 namespace openvdb {
@@ -52,8 +53,8 @@ template<unsigned SIZE, typename T>
 class Mat
 {
 public:
-    typedef T value_type;
-    typedef T ValueType;
+    using value_type = T;
+    using ValueType = T;
     enum SIZE_ { size = SIZE };
 
     // Number of cols, rows, elements
@@ -72,6 +73,15 @@ public:
         }
     }
 
+    Mat& operator=(Mat const& src) {
+        if (&src != this) {
+            for (unsigned i = 0; i < numElements(); ++i) {
+                mm[i] = src.mm[i];
+            }
+        }
+        return *this;
+    }
+
     /// @return string representation of matrix
     /// Since output is multiline, optional indentation argument prefixes
     /// each newline with that much white space. It does not indent
@@ -102,16 +112,17 @@ public:
 
                 // Put a comma after everything except the last
                 if (j) ret.append(", ");
-                ret.append((boost::format("%1%") % mm[(i*SIZE)+j]).str());
+                ret.append(std::to_string(mm[(i*SIZE)+j]));
             }
 
             ret.append("]");
 
             // At the end of every row (except the last)...
-            if (i < SIZE-1 )
-                // ...suffix the row bracket with a comma, newline, and
-                // advance indentation
-                ret.append((boost::format(",\n%1%") % indent).str());
+            if (i < SIZE - 1) {
+                // ...suffix the row bracket with a comma, newline, and advance indentation.
+                ret.append(",\n");
+                ret.append(indent);
+            }
         }
 
         ret.append("]");
@@ -136,6 +147,46 @@ public:
         is.read(reinterpret_cast<char*>(&mm), sizeof(T)*SIZE*SIZE);
     }
 
+    /// Return the maximum of the absolute of all elements in this matrix
+    T absMax() const {
+        T x = static_cast<T>(std::fabs(mm[0]));
+        for (unsigned i = 1; i < numElements(); ++i) {
+            x = std::max(x, static_cast<T>(std::fabs(mm[i])));
+        }
+        return x;
+    }
+
+    /// True if a Nan is present in this matrix
+    bool isNan() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (std::isnan(mm[i])) return true;
+        }
+        return false;
+    }
+
+    /// True if an Inf is present in this matrix
+    bool isInfinite() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (std::isinf(mm[i])) return true;
+        }
+        return false;
+    }
+
+    /// True if no Nan or Inf values are present
+    bool isFinite() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (!std::isfinite(mm[i])) return false;
+        }
+        return true;
+    }
+
+    /// True if all elements are exactly zero
+    bool isZero() const {
+        for (unsigned i = 0; i < numElements(); ++i) {
+            if (!math::isZero(mm[i])) return false;
+        }
+        return true;
+    }
 
 protected:
     T mm[SIZE*SIZE];
@@ -153,7 +204,7 @@ MatType
 rotation(const Quat<typename MatType::value_type> &q,
     typename MatType::value_type eps = static_cast<typename MatType::value_type>(1.0e-8))
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
 
     T qdot(q.dot(q));
     T s(0);
@@ -193,7 +244,7 @@ template<class MatType>
 MatType
 rotation(Axis axis, typename MatType::value_type angle)
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
     T c = static_cast<T>(cos(angle));
     T s = static_cast<T>(sin(angle));
 
@@ -231,7 +282,7 @@ template<class MatType>
 MatType
 rotation(const Vec3<typename MatType::value_type> &_axis, typename MatType::value_type angle)
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
     T txy, txz, tyz, sx, sy, sz;
 
     Vec3<T> axis(_axis.unit());
@@ -316,8 +367,8 @@ eulerAngles(
     RotationOrder rotationOrder,
     typename MatType::value_type eps = static_cast<typename MatType::value_type>(1.0e-8))
 {
-    typedef typename MatType::value_type ValueType;
-    typedef Vec3<ValueType> V;
+    using ValueType = typename MatType::value_type;
+    using V = Vec3<ValueType>;
     ValueType phi, theta, psi;
 
     switch(rotationOrder)
@@ -477,31 +528,33 @@ eulerAngles(
 
 /// @brief Return a rotation matrix that maps @a v1 onto @a v2
 /// about the cross product of @a v1 and @a v2.
-template<class MatType>
-MatType
+/// <a name="rotation_v1_v2"></a>
+template<typename MatType, typename ValueType1, typename ValueType2>
+inline MatType
 rotation(
-    const Vec3<typename MatType::value_type>& _v1,
-    const Vec3<typename MatType::value_type>& _v2,
-    typename MatType::value_type eps=1.0e-8)
+    const Vec3<ValueType1>& _v1,
+    const Vec3<ValueType2>& _v2,
+    typename MatType::value_type eps = static_cast<typename MatType::value_type>(1.0e-8))
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
+
     Vec3<T> v1(_v1);
     Vec3<T> v2(_v2);
 
     // Check if v1 and v2 are unit length
-    if (!isApproxEqual(1.0, v1.dot(v1), eps)) {
+    if (!isApproxEqual(T(1), v1.dot(v1), eps)) {
         v1.normalize();
     }
-    if (!isApproxEqual(1.0, v2.dot(v2), eps)) {
+    if (!isApproxEqual(T(1), v2.dot(v2), eps)) {
         v2.normalize();
     }
 
     Vec3<T> cross;
     cross.cross(v1, v2);
 
-    if (isApproxEqual(cross[0], 0.0, eps) &&
-        isApproxEqual(cross[1], 0.0, eps) &&
-        isApproxEqual(cross[2], 0.0, eps)) {
+    if (isApproxEqual(cross[0], zeroVal<T>(), eps) &&
+        isApproxEqual(cross[1], zeroVal<T>(), eps) &&
+        isApproxEqual(cross[2], zeroVal<T>(), eps)) {
 
 
         // Given two unit vectors v1 and v2 that are nearly parallel, build a
@@ -547,8 +600,8 @@ rotation(
 
         for (int j = 0; j < 3; j++) {
             for (int i = 0; i < 3; i++)
-                result[i][j] =
-                    a * u[i] * u[j] + b * v[i] * v[j] + c * v[j] * u[i];
+                result[i][j] = static_cast<T>(
+                    a * u[i] * u[j] + b * v[i] * v[j] + c * v[j] * u[i]);
         }
         result[0][0] += 1.0;
         result[1][1] += 1.0;
@@ -571,15 +624,15 @@ rotation(
 
         MatType r;
 
-        r[0][0] = c + a0 * cross[0];
-        r[0][1] = a01 + cross[2];
-        r[0][2] = a02 - cross[1],
-        r[1][0] = a01 - cross[2];
-        r[1][1] = c + a1 * cross[1];
-        r[1][2] = a12 + cross[0];
-        r[2][0] = a02 + cross[1];
-        r[2][1] = a12 - cross[0];
-        r[2][2] = c + a2 * cross[2];
+        r[0][0] = static_cast<T>(c + a0 * cross[0]);
+        r[0][1] = static_cast<T>(a01 + cross[2]);
+        r[0][2] = static_cast<T>(a02 - cross[1]);
+        r[1][0] = static_cast<T>(a01 - cross[2]);
+        r[1][1] = static_cast<T>(c + a1 * cross[1]);
+        r[1][2] = static_cast<T>(a12 + cross[0]);
+        r[2][0] = static_cast<T>(a02 + cross[1]);
+        r[2][1] = static_cast<T>(a12 - cross[0]);
+        r[2][2] = static_cast<T>(c + a2 * cross[2]);
 
         if(MatType::numColumns() == 4) padMat4(r);
         return r;
@@ -606,12 +659,12 @@ scale(const Vec3<typename MatType::value_type>& s)
 }
 
 
-/// Return a Vec3 representing the lengths of the passed matrix's upper 3x3's rows.
+/// Return a Vec3 representing the lengths of the passed matrix's upper 3&times;3's rows.
 template<class MatType>
 Vec3<typename MatType::value_type>
 getScale(const MatType &mat)
 {
-    typedef Vec3<typename MatType::value_type> V;
+    using V = Vec3<typename MatType::value_type>;
     return V(
         V(mat[0][0], mat[0][1], mat[0][2]).length(),
         V(mat[1][0], mat[1][1], mat[1][2]).length(),
@@ -619,7 +672,7 @@ getScale(const MatType &mat)
 }
 
 
-/// @brief Return a copy of the given matrix with its upper 3x3 rows normalized.
+/// @brief Return a copy of the given matrix with its upper 3&times;3 rows normalized.
 /// @details This can be geometrically interpreted as a matrix with no scaling
 /// along its major axes.
 template<class MatType>
@@ -631,7 +684,7 @@ unit(const MatType &mat, typename MatType::value_type eps = 1.0e-8)
 }
 
 
-/// @brief Return a copy of the given matrix with its upper 3x3 rows normalized,
+/// @brief Return a copy of the given matrix with its upper 3&times;3 rows normalized,
 /// and return the length of each of these rows in @a scaling.
 /// @details This can be geometrically interpretted as a matrix with no scaling
 /// along its major axes, and the scaling in the input vector
@@ -642,7 +695,7 @@ unit(
     typename MatType::value_type eps,
     Vec3<typename MatType::value_type>& scaling)
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
     MatType result(in);
 
     for (int i(0); i < 3; i++) {
@@ -686,7 +739,7 @@ template<class MatType>
 MatType
 skew(const Vec3<typename MatType::value_type> &skew)
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
 
     MatType r;
     r[0][0] = T(0);      r[0][1] = skew.z();  r[0][2] = -skew.y();
@@ -705,7 +758,7 @@ MatType
 aim(const Vec3<typename MatType::value_type>& direction,
     const Vec3<typename MatType::value_type>& vertical)
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
     Vec3<T> forward(direction.unit());
     Vec3<T> horizontal(vertical.unit().cross(forward).unit());
     Vec3<T> up(forward.cross(horizontal).unit());
@@ -729,7 +782,7 @@ template<class MatType>
 inline MatType
 snapMatBasis(const MatType& source, Axis axis, const Vec3<typename MatType::value_type>& direction)
 {
-    typedef typename MatType::value_type T;
+    using T = typename MatType::value_type;
 
     Vec3<T> unitDir(direction.unit());
     Vec3<T> ourUnitAxis(source.row(axis).unit());
@@ -756,7 +809,7 @@ snapMatBasis(const MatType& source, Axis axis, const Vec3<typename MatType::valu
 }
 
 /// @brief Write 0s along Mat4's last row and column, and a 1 on its diagonal.
-/// @details Useful initialization when we're initializing just the 3x3 block.
+/// @details Useful initialization when we're initializing just the 3&times;3 block.
 template<class MatType>
 static MatType&
 padMat4(MatType& dest)
@@ -771,94 +824,65 @@ padMat4(MatType& dest)
 
 /// @brief Solve for A=B*B, given A.
 /// @details Denman-Beavers square root iteration
-template <typename MatType>
+template<typename MatType>
 inline void
-sqrtSolve(const MatType &aA, MatType &aB, double aTol=0.01)
+sqrtSolve(const MatType& aA, MatType& aB, double aTol=0.01)
 {
-    unsigned int iterations = (unsigned int)(log(aTol)/log(0.5));
-    MatType Y[2];
-    MatType Z[2];
-    MatType invY;
-    MatType invZ;
+    unsigned int iterations = static_cast<unsigned int>(log(aTol)/log(0.5));
 
-    unsigned int current = 0;
-
-    Y[0]=aA;
+    MatType Y[2], Z[2];
+    Y[0] = aA;
     Z[0] = MatType::identity();
 
-    unsigned int iteration;
-    for (iteration=0; iteration<iterations; iteration++)
-    {
+    unsigned int current = 0;
+    for (unsigned int iteration=0; iteration < iterations; iteration++) {
         unsigned int last = current;
         current = !current;
 
-        invY = Y[last].inverse();
-        invZ = Z[last].inverse();
+        MatType invY = Y[last].inverse();
+        MatType invZ = Z[last].inverse();
 
-        Y[current]=0.5*(Y[last]+invZ);
-        Z[current]=0.5*(Z[last]+invY);
+        Y[current] = 0.5 * (Y[last] + invZ);
+        Z[current] = 0.5 * (Z[last] + invY);
     }
-
-    MatType &R = Y[current];
-
-    aB=R;
+    aB = Y[current];
 }
 
 
-template <typename MatType>
+template<typename MatType>
 inline void
-powSolve(const MatType &aA, MatType &aB, double aPower, double aTol=0.01)
+powSolve(const MatType& aA, MatType& aB, double aPower, double aTol=0.01)
 {
-    unsigned int iterations = (unsigned int)(log(aTol)/log(0.5));
+    unsigned int iterations = static_cast<unsigned int>(log(aTol)/log(0.5));
 
-    const bool inverted = ( aPower < 0.0 );
+    const bool inverted = (aPower < 0.0);
+    if (inverted) { aPower = -aPower; }
 
-    if (inverted) {
-        aPower = -aPower;
-    }
-
-    unsigned int whole = (unsigned int)aPower;
+    unsigned int whole = static_cast<unsigned int>(aPower);
     double fraction = aPower - whole;
 
-    MatType R;
-    R = MatType::identity();
-
+    MatType R = MatType::identity();
     MatType partial = aA;
 
     double contribution = 1.0;
-
-    unsigned int iteration;
-
-    for (iteration=0; iteration< iterations; iteration++)
-    {
+    for (unsigned int iteration = 0; iteration < iterations; iteration++) {
         sqrtSolve(partial, partial, aTol);
         contribution *= 0.5;
-
-        if (fraction>=contribution)
-        {
+        if (fraction >= contribution) {
             R *= partial;
-            fraction-=contribution;
+            fraction -= contribution;
         }
     }
 
     partial = aA;
-    while (whole)
-    {
-        if (whole & 1) {
-            R *= partial;
-        }
-        whole>>=1;
-        if(whole) {
-            partial*=partial;
-        }
+    while (whole) {
+        if (whole & 1) { R *= partial; }
+        whole >>= 1;
+        if (whole) { partial *= partial; }
     }
 
-    if (inverted) {
-        aB = R.inverse();
-    }
-    else {
-        aB = R;
-    }
+    if (inverted) { aB = R.inverse(); }
+    else { aB = R; }
 }
 
 
@@ -876,8 +900,8 @@ template<typename MatType>
 inline bool
 isInvertible(const MatType& m)
 {
-    typedef typename MatType::ValueType  value_type;
-    return !isApproxEqual(m.det(), (value_type)0);
+    using ValueType = typename MatType::ValueType;
+    return !isApproxEqual(m.det(), ValueType(0));
 }
 
 
@@ -896,8 +920,8 @@ template<typename MatType>
 inline bool
 isUnitary(const MatType& m)
 {
-    typedef typename MatType::ValueType value_type;
-    if (!isApproxEqual(std::abs(m.det()), value_type(1.0))) return false;
+    using ValueType = typename MatType::ValueType;
+    if (!isApproxEqual(std::abs(m.det()), ValueType(1.0))) return false;
     // check that the matrix transpose is the inverse
     MatType temp = m * m.transpose();
     return temp.eq(MatType::identity());
@@ -914,7 +938,7 @@ isDiagonal(const MatType& mat)
     for (int i = 0; i < n; ++i) {
         for (int j = 0; j < n; ++j) {
             if (i != j) {
-                temp+=std::abs(mat(i,j));
+                temp += std::abs(mat(i,j));
             }
         }
     }
@@ -922,7 +946,7 @@ isDiagonal(const MatType& mat)
 }
 
 
-/// Return the @f$L_\infty@f$ norm of an N x N matrix.
+/// Return the <i>L</i><sub>&infin;</sub> norm of an <i>N</i>&times;<i>N</i> matrix.
 template<typename MatType>
 typename MatType::ValueType
 lInfinityNorm(const MatType& matrix)
@@ -943,7 +967,7 @@ lInfinityNorm(const MatType& matrix)
 }
 
 
-/// Return the @f$L_1@f$ norm of an N x N matrix.
+/// Return the <i>L</i><sub>1</sub> norm of an <i>N</i>&times;<i>N</i> matrix.
 template<typename MatType>
 typename MatType::ValueType
 lOneNorm(const MatType& matrix)
@@ -964,7 +988,7 @@ lOneNorm(const MatType& matrix)
 }
 
 
-/// @brief Decompose an invertible 3x3 matrix into a unitary matrix
+/// @brief Decompose an invertible 3&times;3 matrix into a unitary matrix
 /// followed by a symmetric matrix (positive semi-definite Hermitian),
 /// i.e., M = U * S.
 /// @details If det(U) = 1 it is a rotation, otherwise det(U) = -1,
@@ -1021,6 +1045,6 @@ polarDecomposition(const MatType& input, MatType& unitary,
 
 #endif // OPENVDB_MATH_MAT_HAS_BEEN_INCLUDED
 
-// Copyright (c) 2012-2016 DreamWorks Animation LLC
+// Copyright (c) 2012-2018 DreamWorks Animation LLC
 // All rights reserved. This software is distributed under the
 // Mozilla Public License 2.0 ( http://www.mozilla.org/MPL/2.0/ )
